#
#  Copyright (c) 1991-1993 Regents of the University of California.
#  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: vat.mak,v 1.2 96/02/04 16:02:44 mccanne Exp $ (LBL)
#
APPVER=4.0

!include <win32.mak>

.SUFFIXES: .cc

ALL = vat
all:	$(ALL)

.cc.obj:
	$(cc) $(cdebug) $(cflags) $(cvars) $(CFLAGS) -o $@ -FR$*.sbr -Tp $<

.c.obj:
	$(cc) $(cdebug) $(cflags) $(cvars) $(CFLAGS) -o $@ -FR$*.sbr $<

BSC32= bscmake
BSC32_FLAGS=/nologo /n

MKDEP = ./mkdep

INCLUDE_TK = -I/tcl/include
INCLUDE_TCL = 
INCLUDE_X11 = 
INCLUDE_MISC = 
INCLUDE_GSM = -I../gsm-1.0-pl7/inc

LIB_TK = ../tk4.1a2/win/tk41.lib
LIB_TCL = ../tcl7.5a2/win/tcl75.lib
LIB_GSM = ../gsm-1.0-pl7/lib/gsm.lib

LIB_AUDIO = 
INCLUDE_AUDIO = 
OBJ_AUDIO = audio-win32.obj
OBJ_CRYPT = 
SRC_AUDIO = $(OBJ_AUDIO:.o=.cc) 
LIBS = $(LIB_AUDIO) $(LIB_GSM) $(LIB_TK) $(LIB_TCL) $(guilibs) wsock32.lib winmm.lib
INCLUDES = $(INCLUDE_MISC) $(INCLUDE_AUDIO) $(INCLUDE_TK) $(INCLUDE_TCL) \
	$(INCLUDE_X11) $(MD_INC)
DEFINE = -DSIGRET=void -DSIGARGS=int -DFRAMESIZE=160
BFLAGS = -D_Windows -DWIN32_LEAN_AND_MEAN $(DEFINE) $(INCLUDES)
CFLAGS = $(CCOPT) $(BFLAGS)

#
# Remember to add a dependency if you add any tcl sources here.
#
TCL_VAT = tkerror.tcl entry.tcl ui-main.tcl ui-resource.tcl ui-util.tcl \
	ui-stats.tcl ui-audio.tcl ui-win32.tcl \
	cf-main.tcl cf-network.tcl cf-util.tcl cf-confbus.tcl 
TCL_LIB = tk.tcl

#
# These modules are broken in libg++.  Rather than fix libg++
# every time I install it on a given system, just make them
# here (these sources came from tcl's compat directory)
#
BROKEN_OBJ = 
OBJ0 =	getopt.obj win32.obj
OBJ1 =	inet.obj tonetab.obj adpcm.obj lpc.obj tkStripchart.obj md5c.obj random.obj vat-gsm.obj
OBJ2 =	session.obj session-vat.obj source.obj audio.obj controller.obj \
	crypt.obj crypt-dull.obj filter.obj group-ipc.obj iohandler.obj main.obj \
	net.obj net-ip.obj observe.obj sitebox.obj confbus.obj \
	ss.obj timer.obj idlecallback.obj Tcl.obj Tcl2.obj tkwidget.obj \
	vu.obj wiener.obj media-timer.obj transmitter.obj rate-variable.obj \
	encoder.obj encoder-lpc.obj encoder-adpcm.obj encoder-gsm.obj \
	decoder.obj decoder-lpc.obj decoder-adpcm.obj decoder-gsm.obj
OBJ3 =	tabmulaw.obj tabmix.obj tabsmix.obj tabmulawx.obj
OBJ4 =	$(TCL_VAT:.tcl=.obj) $(TCL_LIB:.tcl=.obj) version.obj
SRC =	$(OBJ1:.obj=.c) $(OBJ2:.obj=.cc) $(OBJ_ARCH:.obj=.cc) $(OBJ_AUDIO:.obj=.cc) \
	$(OBJ_CRYPT:.obj=.cc)
OBJ =	$(OBJ0) $(OBJ1) $(OBJ2) $(OBJ3) $(OBJ4) $(OBJ_ARCH) $(BROKEN_OBJ) \
	$(OBJ_AUDIO) $(OBJ_CRYPT)
SRC3 =	$(OBJ3:.obj=.c)
SBR =	$(OBJ:.obj=.sbr)

vat: vat.exe vat.bsc

vat.exe: $(OBJ)
	$(link) $(ldebug) $(guilflags) -out:$@ $(OBJ) $(LIBS)

vat.bsc : $(OUTDIR)  $(SBR)
    $(BSC32) @<<
  $(BSC32_FLAGS) /o$@ $(SBR)
<<

#
# pass-1 embedded tcl files
#
$(TCL_LIB:.tcl=.obj): tcl2c++.exe
	-del $@
	.\tcl2c++ 1 $(@:.obj=).tcl > $(@:.obj=).cc
	$(cc) $(cdebug) $(cflags) $(cvars) -o $@ -c $(CFLAGS) -FR$*.sbr -Tp $(@:.obj=).cc
	del $(@:.obj=).cc

tk.obj: tk.tcl

#
# pass-2 embedded tcl files
#
$(TCL_VAT:.tcl=.obj): tcl2c++.exe
	-del $@
	.\tcl2c++ 2 $(@:.obj=).tcl > $(@:.obj=).cc
	$(cc) $(cdebug) $(cflags) $(cvars) -o $@ -c $(CFLAGS) -FR$*.sbr -Tp $(@:.obj=).cc
	del $(@:.obj=).cc

tkerror.obj: tkerror.tcl
entry.obj: entry.tcl
ui-main.obj: ui-main.tcl
ui-audio.obj: ui-audio.tcl
ui-util.obj: ui-util.tcl
ui-win32.obj: ui-win32.tcl
ui-stats.obj: ui-stats.tcl
ui-resource.obj: ui-resource.tcl
cf-main.obj: cf-main.tcl
cf-network.obj: cf-network.tcl
cf-util.obj: cf-util.tcl
cf-confbus.obj: cf-confbus.tcl

version.c: VERSION
	-del version.c
	cat VERSION | sed 's/.*/char version[] = "&";/' > version.c

$(OBJ3): $(SRC3)
	$(cc) $(cdebug) $(cflags) $(cvars) $(RFLAG) -o $@ -FR$*.sbr $*.c

mktab.exe: mktab.obj getopt.obj
	-del $@
	$(cc) /nologo -o $@ $(CFLAGS) mktab.obj getopt.obj

$(SRC3): mktab.exe
	-del $(SRC3)
	.\mktab -mulaw > tabmulaw.c
	.\mktab -sum > tabmix.c
	.\mktab -scale > tabsmix.c
	.\mktab -muX > tabmulawx.c
	-del mktab

vat-gsm.obj: vat-gsm.c
	$(cc) $(cdebug) $(cflags) $(cvars) $(INCLUDE_GSM) $(CFLAGS) -o $@ -FR$*.sbr $*.c

tar: force
	tar=vatbin-`cat VERSION`-@V_TAR_TARGET@.tar.gz ; \
	rm -f $$tar ; \
	@V_TARCMD@ - README CHANGES.html vat.1 @V_TAR_EXTRA@ $(ALL) | \
		gzip -c > $$tar

srctar: force
	@cwd=`pwd` ; dir=`basename $$cwd` ; \
	    name=vat-`cat VERSION | tr A-Z a-z` ; \
	    tar=vatsrc-`cat VERSION`.tar.gz ; \
	    list="" ; \
	    for i in `cat FILES` ; do list="$$list $$name/$$i" ; done; \
	    echo \
	    "(rm -f $$tar; cd .. ; ln -s $$dir $$name)" ; \
	     (rm -f $$tar; cd .. ; ln -s $$dir $$name) ; \
	    echo \
	    "(cd .. ; tar cfh $$tar [lots of files])" ; \
	     (cd .. ; tar cfh - $$list) | gzip -c > $$tar ; \
	    echo \
	    "rm ../$$name; chmod 444 $$tar" ;  \
	     rm ../$$name; chmod 444 $$tar

install: force
	if [ -d /usr/local/xbin ]; then
	    mv /usr/local/xbin/vat /usr/local/bin/xvat-
	    cp vat /usr/local/xbin/vat
	    chown bin.bin /usr/local/xbin/vat
	    chmod 555 /usr/local/xbin/vat
	    rm -f /usr/local/bin/vat
	    ln -s ../xbin/vat /usr/local/bin/vat
	else
	    mv /usr/local/bin/vat /usr/local/bin/vat-
	    cp vat /usr/local/bin/vat
	    chown bin.bin /usr/local/bin/vat
	    chmod 555 /usr/local/bin/vat
	fi

install-man: force
	cp vat.1 /usr/local/man/man1/vat.1
	chown bin.bin /usr/local/man/man1/vat.1
	chmod 444 /usr/local/man/man1/vat.1

CLEAN = vat vat.dyn core core.vat tcl2c++ config.log config.cache
clean:
	rm -f $(CLEAN) $(OBJ1) $(OBJ2) $(OBJ4) $(OBJ_AUDIO) $(OBJ_CRYPT)
		
c++-clean:
	rm -f $(OBJ2) $(OBJ_AUDIO)

realclean:
	rm -f $(CLEAN) $(OBJ) tags \
		mktab tabmix.c tabmulaw.c tabsmix.c
		
force:

tags:	$(SRC)
	./systags.sh

depend:
	./mkdep $(INCLUDES) $(DEFINE) $(INCLUDE_GSM) $(SRC)
